var exec = require('child_process').exec;
var fs = require('fs');
var path = require('path');
var async = require('async');
var gm = require('gm');
var shellQuote = require('shell-quote');
var mkdirp = require('mkdirp');
var tmp = require('tmp');

function ImageDiff() {
}
ImageDiff.getImageSize = function (filepath, cb) {
  // TODO: This could be done via pngjs but stick to imagemagick for now
  fs.stat(filepath, function (err, stats) {
    // If the file does not exist, callback with info
    if (err) {
      if (err.code === 'ENOENT') {
        return cb(null, null);
      } else {
        return cb(err);
      }
    }

    gm(filepath).size(function (err, value) {
      if (err) {
        return cb(err);
      }
      cb(null, value);
    });
  });
};
ImageDiff.createDiff = function (options, cb) {
  var diffCmd = shellQuote.quote([
    'compare',
    '-verbose',
    // TODO: metric and highlight could become constructor options
    '-metric', 'RMSE',
    '-highlight-color', 'RED',
    '-compose', 'Src',
    options.actualPath,
    options.expectedPath,
    options.diffPath
  ]);
  exec(diffCmd, function processDiffOutput (err, stdout, stderr) {
    // If we failed with no info, callback
    if (err && !stderr) {
      return cb(err);
    }

    // Callback with pass/fail
    // TODO: This is not very multi-lengual =(
    return cb(null, stderr.indexOf('all: 0 (0)') !== -1);
  });
};
ImageDiff.prototype = {
  diff: function (options, callback) {
    // TODO: Break this down more...
    var actualPath = options.actualImage;
    var expectedPath = options.expectedImage;
    var diffPath = options.diffImage;

    // Assert our options are passed in
    if (!actualPath) {
      return process.nextTick(function () {
        callback(new Error('`options.actualPath` was not passed to `image-diff`'));
      });
    }
    if (!expectedPath) {
      return process.nextTick(function () {
        callback(new Error('`options.expectedPath` was not passed to `image-diff`'));
      });
    }
    if (!diffPath) {
      return process.nextTick(function () {
        callback(new Error('`options.diffPath` was not passed to `image-diff`'));
      });
    }

    var actualTmpPath;
    var expectedTmpPath;
    var imagesAreSame;
    async.waterfall([
      function assertActualPathExists (cb) {
        fs.exists(actualPath, function handleActualExists (actualExists) {
          if (actualExists) {
            cb();
          } else {
            cb(new Error('`image-diff` expected "' + actualPath + '" to exist but it didn\'t'));
          }
        });
      },
      function collectImageSizes (cb) {
        // Collect the images sizes
        async.map([actualPath, expectedPath], ImageDiff.getImageSize, cb);
      },
      function resizeImages (sizes, cb) {
        // Find the maximum dimensions
        var actualSize = sizes[0];
        var expectedSize = sizes[1] || {doesNotExist: true, height: 0, width: 0};
        var maxHeight = Math.max(actualSize.height, expectedSize.height);
        var maxWidth = Math.max(actualSize.width, expectedSize.width);

        // Resize both images
        async.parallel([
          function resizeActualImage (cb) {
            // Get a temporary filepath
            tmp.tmpName({postfix: '.png'}, function (err, filepath) {
              // If there was an error, callback
              if (err) {
                return cb(err);
              }

              // Otherwise, resize the image
              actualTmpPath = filepath;
              gm(actualPath).resize(maxWidth, maxHeight).write(actualTmpPath, cb);
            });
          },
          function resizeExpectedImage (cb) {
            tmp.tmpName({postfix: '.png'}, function (err, filepath) {
              // If there was an error, callback
              if (err) {
                return cb(err);
              }

              // If there was no expected image, create a transparent image to compare against
              expectedTmpPath = filepath;
              if (expectedSize.doesNotExist) {
                gm(maxWidth, maxHeight, 'transparent').write(expectedTmpPath, cb);
              // Otherwise, resize the image
              } else {
                gm(expectedPath).resize(maxWidth, maxHeight).write(expectedTmpPath, cb);
              }
            });
          }
        ], cb);
      },
      function createDiffDirectory (/*..., cb*/) {
        var cb = [].slice.call(arguments, -1)[0];
        mkdirp(path.dirname(diffPath), function (err) {
          cb(err);
        });
      },
      function createDiff (cb) {
        ImageDiff.createDiff({
          actualPath: actualTmpPath,
          expectedPath: expectedTmpPath,
          diffPath: diffPath
        }, function saveResult (err, _imagesAreSame) {
          imagesAreSame = _imagesAreSame;
          cb(err);
        });
      }
    ], function cleanup (err) {
      // Clean up the temporary files
      var cleanupPaths = [actualTmpPath, expectedTmpPath].filter(function (filepath) {
        return !!filepath;
      });
      async.forEach(cleanupPaths, function cleanupFile (filepath, cb) {
        fs.unlink(filepath, cb);
      }, function callOriginalCallback (_err) {
        // Callback with the imagesAreSame
        callback(err, imagesAreSame);
      });
    });
  }
};

// Create helper utility
function diffImages(options, cb) {
  var differ = new ImageDiff();
  differ.diff(options, cb);
}

// Export the original class and helper
diffImages.ImageDiff = ImageDiff;
module.exports = diffImages;
